const readline = require('readline');
const read = require('fs-readdir-recursive');
const fs = require('fs');

const reader = readline.createInterface({
  input: process.stdin,
  output: process.stdout,
});

console.log(
  '\nInnan vi forsätter med uppgraderingen måste du se' +
    '\ntill att du står i rotmappen av projektet du vill uppgradera.'
);
console.log('\nDessutom att du inte har några ändringar som inte är committade' + '\ni din branch.');

console.log(
  '\nObs! att detta script endast går igenom och uppdaterar htmlfilerna i ditt projekt,\n' +
    'finns det ändra typer av filer som innehåller kod från designsystem som t.ex. ts, tsx, js, less, scss etc. så får ni manuellt justera dessa'
);

reader.question('\nVill du fortsätta? (y/n)', async function (svar) {
  if (svar.toLowerCase() == 'y') {
    console.log('Letar igenom alla html filer.\n');
    const htmlFiles = read(process.cwd())
      .filter(item => item.endsWith('.html'))
      .filter(item => !item.startsWith('node_modules'))
      .filter(item => !item.startsWith('dist'));

    let replaceCheckmarkFound = false;
    for (const htmlFile of htmlFiles) {
      lookingHeadersMissingAriaLabel(htmlFile);
      lookingForTomOptions(htmlFile);
      lookingInlineTabellHeader(htmlFile);
      lookingFullwidthTabellHeader(htmlFile);
      lookingResponsiveTabellContainer(htmlFile);
      lookingBanners(htmlFile);
      lookingModalCloseBtn(htmlFile);
      replaceCheckmarkFound = lookingCheckboxmark(htmlFile) || replaceCheckmarkFound;
    }
    if (replaceCheckmarkFound) {
      console.log('Ersätt befintlig SVG under klassen "c-checkbox__checkbox" med: \n' + newCheckmarkSVG)
    }

    console.log(`
    Manuella åtgärder
    1. Vi har ny komponent <jv-single-select> för single select komponenter.
       Du behöver manuellt byta alla <jv-dynamisk-input> med single select.
    2. c-input__wrapper ska byta namn till c-select__wrapper för select komponenter.
    3. Om du använder single-select html-css only, du behöver lägg till en onchange method.
       t.ex.<select id="1" onchange="this.dataset.empty = this.value == ''">
       `);

    await reader.close();
  } else {
    await reader.close();
  }
});

const lookingBanners = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/class="c-banner__text*/i.test(data)) {
    console.log(
      'Finns en eller banners där ni behöver lägga in klassen c-banner-p på alla paragrafer\n' + htmlFile + '\n');
    return true;
  }
  return false;
};

const lookingCheckboxmark = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/class="c-checkbox__checkbox*/i.test(data)) {
    console.log(
      'Finns en eller flera checkboxar där ni behöver byta ut svg i filen\n' + htmlFile + '\n');
    return true;
  }
  return false;
};

const newCheckmarkSVG = '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" x="0px" y="0px" \nfocusable="false" width="0.4375em" height="0.4375em" viewBox="0 0 78.369 78.369"><g>\n'
  + '<path d="M78.049,19.015L29.458,67.606c-0.428,0.428-1.121,\n0.428-1.548,0L0.32,40.015c-0.427-0.426-0.427-1.119,\n0-1.547l6.704-6.704c0.428-0.427,1.121-0.427,'
  + '\n1.548,0l20.113,20.112l41.113-41.113c0.429-0.427,1.12-0.427,\n1.548,0l6.703,6.704C78.477,17.894,78.477,18.586,78.049,19.015z"/>\n</g></svg>';

const lookingInlineTabellHeader = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/class=".*inline-header*/i.test(data)) {
    console.log(
      'OBS! Har hittat en eller flera .inline-header tabell klass ' +
        'som behöver ersättas av .c-table__header--col klassen manuellt i filen ' +
        htmlFile
    );
    return true;
  }
  return false;
};

const lookingFullwidthTabellHeader = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/class=".*full-width-header*/i.test(data)) {
    console.log(
      'OBS! Har hittat en eller flera .full-width-header tabell klass ' +
        'som behöver ersättas av .c-table__header--fullwidth klassen manuellt i filen ' +
        htmlFile
    );
    return true;
  }
  return false;
};

const lookingResponsiveTabellContainer = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/class=".*c-responsive-table-container*/i.test(data)) {
    data = data.replace(/c-responsive-table-container/g, `c-table--responsive-container`);
    writeToFile(
      htmlFile,
      data,
      '\nHittat .c-responsive-table-container och ändrat till c-table--responsive-container i ' + htmlFile
    );
  }
};

const lookingModalCloseBtn = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/class=".*modal-close-btn*/i.test(data)) {
    data = data.replace(/modal-close-btn/g, `mb-0`);
    writeToFile(
      htmlFile,
      data,
      '\nHittat .modal-close-btn och ändrat till mb-0 i ' + htmlFile
    );
  }
};

const lookingHeadersMissingAriaLabel = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  let nbrOfString = 0;
  if (/<a.*class="c-header__content--title__link"/i.test(data)) {
    ({ data, nbrOfString } = findAndUpdate(data, nbrOfString, /<a.*class="c-header__content--title__link".*>/gm));
    if (nbrOfString > 0) {
      writeToFile(
        htmlFile,
        data,
        '\nHittat och lagt till aria-label på ' + nbrOfString + 'st länkar i sidhuvudet, ' + htmlFile
      );
    }
  }
};

const lookingForTomOptions = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/<option><\/option>/i.test(data)) {
    data = data.replace(/<option><\/option>/g, `<option value="">-- Inget valt --</option>`);
    writeToFile(htmlFile, data, '\nHittat och ersatt ' + 'tom option med rätt html i filen ' + htmlFile);
  }
};

function findAndUpdate(data, nbrOfString, regexp) {
  const matchingArray = data.match(regexp);

  if (!matchingArray) {
    return { data, nbrOfString };
  }
  for (const matchingString of matchingArray) {
    if (!matchingString.includes('aria-label="')) {
      data = replaceAll(
        data,
        matchingString,
        matchingString.replace(
          'class="c-header__content--title__link"',
          'class="c-header__content--title__link" aria-label="Jordbruksverket, till startsidan"'
        )
      );
      nbrOfString = nbrOfString + 1;
    }
  }
  return { data, nbrOfString };
}

function writeToFile(htmlFile, data, message) {
  fs.writeFileSync(htmlFile, data, 'utf8');
  console.log(message);
}

reader.on('close', function () {
  console.log('\nAvslutar!');
  process.exit(0);
});

function replaceAll(string, match, replaceString) {
  return string.split(match).join(replaceString);
}
